from sqlalchemy import select, insert, delete, ScalarResult
from sqlalchemy.orm import Session
from sqlalchemy.exc import ResourceClosedError

from ..database.db import async_session, engine
from ..database.models import UnreceivedMsg


async def insert_unreceived_msg(
    msg_id: str,
    event: str,
    type: str,
    sender_id: str,
    receiver_id: str,
    text: str,
    attachments: list[str],
    date_time: str,
    is_show_time,
    group_chat_id: str = None,
    nickname: str = None,
    remark_in_group_chat: str = None,
    avatar: str = None,
):
    session = async_session()
    try:
        await session.execute(
            insert(UnreceivedMsg).values(
                msg_id=msg_id,
                event=event,
                type=type,
                sender_id=sender_id,
                receiver_id=receiver_id,
                group_chat_id=group_chat_id,
                nickname=nickname,
                remark_in_group_chat=remark_in_group_chat,
                avatar=avatar,
                text=text,
                attachments=attachments,
                date_time=date_time,
                is_show_time=is_show_time,
            )
        )
        await session.commit()
    except Exception as e:
        print(f"Insert Unreceived msg fail with error: {e}")
        raise e
    finally:
        await session.close()


def select_msgs(receiver_id: str) -> list[UnreceivedMsg]:
    session = Session(engine)
    try:
        res: ScalarResult[UnreceivedMsg] = session.scalars(
            select(UnreceivedMsg)
            .where(UnreceivedMsg.receiver_id == receiver_id)
            .order_by(UnreceivedMsg.msg_id.asc())
        )
        msgs = list(res.all())
        session.close()
        return msgs
    except ResourceClosedError as e1:
        print(e1)
        return []
    except Exception as e2:
        print(e2)
        return []
    finally:
        session.close()


async def delete_and_return_msgs(receiver_id: str) -> list[UnreceivedMsg]:
    session = async_session()
    try:
        res: ScalarResult[UnreceivedMsg] = await session.execute(
            delete(UnreceivedMsg)
            .where(UnreceivedMsg.receiver_id == receiver_id)
            .returning()
        )
        msgs = res.all()
        await session.commit()
        await session.close()
        return list(msgs)
    except ResourceClosedError as e1:
        print(e1)
        raise e1
    except Exception as e2:
        print(e2)
        raise e2
    finally:
        await session.close()


def delete_msgs(receiver_id: str):
    session = Session(engine)
    try:
        session.execute(
            delete(UnreceivedMsg).where(UnreceivedMsg.receiver_id == receiver_id)
        )
        session.commit()
    except Exception as e:
        print(e)
    finally:
        session.close()
