import os
import random
import array
import math
from typing import Literal, Tuple
from pathlib import Path
from datetime import datetime
from zipfile import ZipFile

import aiofiles

alphabet = [
    "a",
    "b",
    "c",
    "d",
    "e",
    "f",
    "g",
    "h",
    "i",
    "j",
    "k",
    "l",
    "m",
    "n",
    "o",
    "p",
    "q",
    "r",
    "s",
    "t",
    "u",
    "v",
    "w",
    "x",
    "y",
    "z",
]

CHUNK_SIZE = 1024 * 1024 * 1


def create_avatar_dir(type: Literal["user", "group_chat"], dir_name: str) -> Path:
    if type == "user":
        avatar_dir_path = Path(os.getcwd()) / "static" / dir_name / "user"
        if not avatar_dir_path.exists():
            avatar_dir_path.mkdir()
        return avatar_dir_path
    else:
        avatar_dir_path = Path(os.getcwd()) / "static" / dir_name / "group_chat"
        if not avatar_dir_path.exists():
            avatar_dir_path.mkdir()
        return avatar_dir_path


def create_avatar_filename() -> str:
    timestamp = int(datetime.now().timestamp())
    random_prefix = "".join(random.choices(alphabet, k=8))
    return f"{random_prefix}{timestamp}.png"


def create_zip_file(filenames: list[str], file_type: Literal["avatars"]) -> Path:
    zip_filename = f"temp_{file_type}_{round(datetime.now().timestamp())}.zip"
    zip_dir = Path(os.getcwd()) / "static" / "temp"
    file_dir = Path(os.getcwd()) / "static" / file_type
    if not zip_dir.exists():
        zip_dir.mkdir(parents=True)

    with ZipFile(zip_dir / zip_filename, "w") as zip_file:
        for filename in filenames:
            zip_file.write(file_dir / filename, arcname=filename)

    return zip_dir / zip_filename


async def write_chat_file(
    msg: dict,
    file_type: Literal["image", "video"],
):
    filename = msg["filename"]
    total_chunk_num = msg["totalChunkNum"]
    temp_filename = msg["tempFilename"]

    if (Path(os.getcwd()) / "static" / "chat" / file_type / filename).exists():
        return
    match file_type:
        case "image":
            sub_dir = filename.split("/")[0]
            chat_image_dir = Path(os.getcwd()) / "static" / "chat" / "images"
            if not (chat_image_dir / sub_dir).exists():
                os.makedirs(chat_image_dir / sub_dir)
            temp_image_dir = Path(os.getcwd()) / "static" / "temp"
            if not (temp_image_dir / sub_dir).exists():
                os.makedirs(temp_image_dir / sub_dir)

            if total_chunk_num == 1:
                async with aiofiles.open(chat_image_dir / filename, "wb") as file:
                    await file.write(bytearray(msg["bytes"]))
                return

            if not (temp_image_dir.parent / temp_filename).exists():
                try:
                    async with aiofiles.open(
                        temp_image_dir.parent / temp_filename, "wb"
                    ) as f:
                        await f.write(bytearray(msg["bytes"]))
                except Exception as e:
                    print(f"write temp file fail with error: {e}")

            temp_file_path_exist: list[bool] = [
                (temp_image_dir / f"{filename}-{total_chunk_num}-{i}").exists()
                for i in range(total_chunk_num)
            ]

            # assemble the file when all chunk is arrived
            if all(temp_file_path_exist):
                async with aiofiles.open(chat_image_dir / filename, "ab+") as file:
                    for i in range(total_chunk_num):
                        async with aiofiles.open(
                            temp_image_dir / f"{filename}-{total_chunk_num}-{i}", "rb"
                        ) as chunk_file:
                            bytes_content = await chunk_file.read()
                        await file.write(bytes_content)
                        os.remove(temp_image_dir / f"{filename}-{total_chunk_num}-{i}")
        case "video":
            pass


def read_chat_file(filename: str) -> Tuple[int, int, list[int]]:
    file_suffix: str = filename.split(".")[1]

    if file_suffix == "png":
        file_type = "images"
    else:
        file_type = "videos"

    p = Path(os.getcwd()) / "static" / "chat" / file_type / filename
    file_size = p.stat().st_size
    total_chunk_num = math.ceil(file_size / CHUNK_SIZE)

    if not p.exists():
        yield -1, 0, []
    else:
        file = open(p, "rb")
        for i in range(total_chunk_num):
            byte_data = file.read(CHUNK_SIZE)
            yield i, total_chunk_num, array.array("B", byte_data).tolist()

        file.close()


async def async_read_chat_file(file_path: str) -> Tuple[int, int, list[int]]:
    file_suffix: str = file_path.split(".")[1]

    if file_suffix == "png":
        file_type = "images"
    else:
        file_type = "videos"

    p = Path(os.getcwd()) / "static" / "chat" / file_type / file_path
    file_size = p.stat().st_size
    total_chunk_num = math.ceil(file_size / CHUNK_SIZE)

    if not p.exists():
        yield -1, 0, []
    else:
        async with aiofiles.open(p, "rb") as f:
            for i in range(total_chunk_num):
                byte_data = await f.read(CHUNK_SIZE)
                yield i, total_chunk_num, array.array("B", byte_data).tolist()
