from fastapi import WebSocket

from ..crud.unreceived_msg_crud import insert_unreceived_msg
from ..utils.static_file import write_chat_file


class WebSocketManager:
    def __init__(self):
        self.active_socket: dict[str, WebSocket] = {}

    async def connect(self, id: str, websocket: WebSocket):
        print("建立连接中...........")
        await websocket.accept()
        self.active_socket[id] = websocket

    async def reconnect(self, id: str, websocket: WebSocket):
        print("重连中...........")
        self.disconnect(id)
        await websocket.accept()
        self.active_socket[id] = websocket

    def disconnect(self, user_id: str):
        if self.active_socket.get(user_id):
            del self.active_socket[user_id]

    def disconnect_all(self):
        self.active_socket.clear()

    async def send_to_another(self, another: str, msg: dict):
        if self.active_socket.get(another):
            socket = self.active_socket.get(another)
            await socket.send_json(msg)

        else:
            if msg["event"] == "friend-chat-msg":
                await insert_unreceived_msg(
                    msg["msgId"],
                    msg["event"],
                    msg["type"],
                    msg["senderId"],
                    msg["receiverId"],
                    msg["text"],
                    msg["attachments"],
                    msg["dateTime"],
                    msg["isShowTime"],
                )
        if msg["event"] == "chat-image":
            await write_chat_file(msg, "image")
            if self.active_socket.get(msg["senderId"]):
                ws = self.active_socket.get(msg["senderId"])
                await ws.send_json(
                    {
                        "event": "chat-image-send-ok",
                        "chatType": 0,
                        "receiverId": another,
                        "currentChunkNum": msg["currentChunkNum"],
                        "totalChunkNum": msg["totalChunkNum"],
                        "filename": msg["filename"],
                    }
                )

    async def broadcast(
        self,
        receiver_ids: list[str],
        msg: dict,
    ):
        for receiver_id in receiver_ids:
            socket = self.active_socket.get(receiver_id)
            if socket:
                await socket.send_json(msg)
            else:
                if msg["event"] == "group-chat-msg":
                    await insert_unreceived_msg(
                        msg["msgId"],
                        msg["event"],
                        msg["type"],
                        msg["senderId"],
                        receiver_id,
                        msg["text"],
                        msg["attachments"],
                        msg["dateTime"],
                        msg["isShowTime"],
                        group_chat_id=msg["groupChatId"],
                        nickname=msg["nickname"],
                        remark_in_group_chat=msg["remarkInGroupChat"],
                        avatar=msg["avatar"],
                    )
            if msg["event"] == "chat-image":
                await write_chat_file(msg, "image")
                if self.active_socket.get(msg["senderId"]):
                    await self.active_socket["senderId"].send_json(
                        {
                            "event": "chat-image-send-ok",
                            "chatType": 1,
                            "receiverIds": receiver_ids,
                            "currentChunkNum": msg["currentChunkNum"],
                            "totalChunkNum": msg["totalChunkNum"],
                            "filename": msg["filename"],
                        }
                    )
